# http://www.python.org/peps/pep-0355.html
# http://hrnjad.net/src/6/scriptutil.py

__all__ = ['PRUNE', 'find', 'pyFiles', 'suffixFilter', 'compareFiles']

import os

PRUNE = object()

def find(paths=(), filter=lambda f:True):
    '''Walks over list of files and directories doing recursive walk
       in case of directory.
    '''
    from os.path import abspath, normpath, join, isdir
    from iterutils import imap, chain, sorted

    if isinstance(paths, basestring):
        paths = [paths]
    if not paths:
        #paths = [os.getcwd()]
        paths = ['.']

    def _walk(path):
        r = filter(path)
        if r is PRUNE:
            return
        if r:
            yield path
        if not isdir(path):
            return
        for f in sorted(os.listdir(path)):
            for i in _walk(join(path, f)):
                yield i

    for path in paths:
        #for f in _walk(abspath(normpath(path))):
        for f in _walk(path):
            yield f


def pyFiles(f):
    if f.endswith(os.sep + '.svn'):
        return PRUNE
    if f.endswith(os.sep + 'CVS'):
        return PRUNE
    if f.endswith('.py'):
        return True
    else:
        return False


def suffixFilter(*suff):
    def findFilter(f):
        if f.endswith(os.sep + '.svn'):
            return PRUNE
        if f.endswith(os.sep + 'CVS'):
            return PRUNE
        for s in suff:
            if f.endswith(s):
                return True
        return False
    return findFilter




def compareFiles(src, dest):

    from os.path import isfile, getsize

    if not isfile(src) or not isfile(dest):
        return False

    block_size = 1024 * 128

    if getsize(src) == getsize(dest):
        sfd, dfd = open(src), open(dest)
        while True:
            sbuf, dbuf = sfd.read(block_size), dfd.read(block_size)
            if sbuf != dbuf:
                return False
            if len(sbuf) < block_size:
                return True
    else:
        return False
