# http://users.forthnet.gr/ath/chrisgeorgiou/python/mailloganalyse.py

import re
from timeutils import Datetime
from itertools import izip

__all__ = ['LogSyntaxError', 'Parser', 'SendmailParser']


class LogSyntaxError(ValueError):
    pass


class Parser(object):

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def parseLine(self, line):
        lineSplit  = line.strip().split(None, 5)
        if len(lineSplit) < 5:
            raise LogSyntaxError('invalid log line', line)
        if len(lineSplit) < 6:
            logMsg = ''
        else:
            logMsg = lineSplit[5]
        month, day, time, host, process = lineSplit[:5]
        try:
            timestamp = Datetime('%s %s %s' % (month, day, time))
        except:
            raise LogSyntaxError(
                'invalid timestamp', '%s %s %s' % (month, day, time))
        return timestamp, host, process, logMsg



class SendmailParser(Parser):

    mailRecSplit = re.compile(', ([a-z]+=)').split

    def parseLine(self, line):
        timestamp, host, process, d = super(SendmailParser,self).parseLine(line)
        msgId, logMsg = d.split(None,1)
        msgId = msgId[:-1]
        logRec = self.mailRecSplit(', ' + logMsg)[1:]
        if len(logRec) % 2 != 0:
            raise LogSyntaxError('invalid mail log record')
        it = iter(logRec)
        logRec = dict([(k[:-1].lower(),v) for (k,v) in izip(it,it)])
        return timestamp, host, process, msgId, logRec


class PostfixParser(SendmailParser):

    def parseLine(self, line):
        timestamp, host, process, d = super(SendmailParser,self).parseLine(line)




if __name__ == '__main__':

    simpleLog = [
    """Aug  8 11:51:25 trautenberg sm-mta[18885]: j789pOxf018885: from=<owner-vsnet-l@LIST.EMWAC.CZ>, size=5790, class=-30, nrcpts=1, msgid=<20050808115117.7ajvn77zgkgoowgc@webmail.nextra.sk>, bodytype=8BITMIME, proto=ESMTP, daemon=MTA, relay=nms.emwac.cz [195.113.117.1]""",
    """Aug  8 11:51:25 trautenberg sm-mta[18890]: j789pOxf018885: to=<vlk@DAT.CZ>, delay=00:00:00, xdelay=00:00:00, mailer=local, pri=89941, dsn=2.0.0, stat=Sent""",
    """Aug  8 11:51:25 trautenberg""",
    """XXX  8 11:51:25 trautenberg sm-mta""",
    ]

    p = SendmailParser()
    for l in simpleLog:
        try:
            x = p.parseLine(l)
            for k,v in x[-1].items():
                print '%s=%s' % (k, repr(v))
        except LogSyntaxError, e:
            print e
        print
