import thread
from logging import Handler, Formatter, LogRecord
from logging import FATAL, CRITICAL, ERROR, WARNING, WARN, INFO, DEBUG
from logging.handlers import SMTPHandler


class Memory(Handler):

    capacity = 0
    buffer = []
    clock = thread.allocate_lock()

    def __init__(self, level=WARNING, formatter=None):
        Handler.__init__(self, level=level)
        if formatter is not None:
            self.setFormatter(formatter)

    def _getLog (klass, level=DEBUG) :
        klass.clock.acquire()
        try :
            b = []
            for r,m in klass.buffer :
                if r.levelno >= level :
                    b.append(m)
            return b
        finally :
            klass.clock.release()
    getLog = classmethod(_getLog)

    def _clearLog (klass) :
        klass.clock.acquire()
        try :
            klass.buffer[:] = []
        finally :
            klass.clock.release()
    clearLog = classmethod(_clearLog)

    def _setCapacity (klass, cap) :
        klass.capacity = cap
    setCapacity = classmethod(_setCapacity)

    def emit(self, r):
        t = self.format(r)
        b = self.__class__.buffer
        if self.capacity and len(b) >= self.capacity :
            del b[0]
        b.append((r, t))
        r.exc_info = None


LOGREC_ARGS = LogRecord.__init__.im_func.func_code.co_argcount

class Request(Handler):

    def __init__(
        self, level=DEBUG, emitLevel=ERROR, emitHandler=None, accumulate=True,
        formatter=None,
    ):
        Handler.__init__(self, level=level)
        if formatter is not None:
            self.setFormatter(formatter)
        from solmundr.components import getScope
        from solmundr.web import IRequest
        self.get_req = lambda: getScope(IRequest)
        self.emitLevel = emitLevel
        self.emitHandler = emitHandler
        self.accumulate = accumulate

    def emit(self, r):
        req = self.get_req()
        if r.levelno < self.emitLevel:
            if req is not None:
                req.req_log.append(r)
        else:
            if self.emitHandler is not None:
                if req is not None:
                    req.req_log.append(r)
                    if self.accumulate:
                        t = '\n'.join(
                            self.format(o).rstrip() for o in req.req_log
                        )
                        if LOGREC_ARGS == 8:
                            n = LogRecord(
                                r.name, r.levelno, r.pathname, r.lineno,
                                t, (), None)
                        else:
                            n = LogRecord(
                                r.name, r.levelno, r.pathname, r.lineno,
                                t, (), None, None)
                        self.emitHandler.emit(n)
                    else:
                        for o in req.req_log:
                            self.emitHandler.emit(o)
                    req.req_log[:] = []
                else:
                    self.emitHandler(r)



# Enhanced http://cheeseshop.python.org/pypi/mailinglogger/3.0.0

class Sendmail(SMTPHandler):

    def __init__(self, toaddr, subject, mailhost='localhost', fromaddr=None):
        if fromaddr is None:
            import socket, pwd
            fromaddr = '%s@%s' % (
                pwd.getpwuid(os.getuid()).pw_name,
                socket.getfqdn()
            )
        if isinstance(toaddr, basestring):
            toaddr = toaddr,
        SMTPHandler.__init__(
            self, mailhost, fromaddr, toaddr, subject
        )


class Null(Handler):

    def emit(self, r):
        pass
