__all__ = ['AttrProxy', 'ComputedProperty', 'LateBindingProperty']


class AttrProxy(object):

    def __init__(self, attrPath):
        self.attrPath = attrPath.replace('/','.').split('.')

    def __get__(self, obj, typ):
        if obj is None:
            return self
        return reduce(getattr, self.attrPath, obj)

    def __set__(self, obj, value):
        obj = reduce(getattr, self.attrPath[:-1], obj)
        setattr(obj, self.attrPath[-1], value)

    def __delete__(self, obj):
        obj = reduce(getattr, self.attrPath[:-1], obj)
        delattr(obj, self.attrPath[-1])

    def __repr__(self):
        return 'AttrProxy(%r)' % '.'.join(self.attrPath)



class ComputedProperty(object):

    def __init__(self, factory, attrName=None):
        self.factory = factory
        self.attr = attrName or self

    def __get__(self, obj, typ):
        if obj is None:
            return self
        odict = obj.__dict__
        value = odict.get(self.attr, self)
        if value is self:
            value = odict[self.attr] = self.factory(obj)
        return value

    def __set__(self, obj, value):
        obj.__dict__[self.attr] = value

    def __delete__(self, obj):
        del obj.__dict__[self.attr]







def update_meta(self, other):
    #self.__name__ = other.__name__
    self.__doc__ = other.__doc__
    self.__dict__.update(other.__dict__)
    return self


class LateBindingProperty (property) :
    '''Property that allows the getter and setter methods associated
       with the property to be overridden in subclasses.

       Usage:

        class C (object) :
            def getx(self) :
                print 'C.getx'
                return self._x
            x = LateBindingProperty(getx)

        class D(C) :
            def getx(self) :
                print 'D.getx'
                super(D, self).getx()

       From: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/408713
    '''

    def __new__(cls, fget=None, fset=None, fdel=None, doc=None):

        if fget is not None:
            def __get__(obj, objtype=None, name=fget.__name__):
                return getattr(obj, name)()
            fget = update_meta(__get__, fget)

        if fset is not None:
            def __set__(obj, value, name=fset.__name__):
                return getattr(obj, name)(value)
            fset = update_meta(__set__, fset)

        if fdel is not None:
            def __delete__(obj, name=fdel.__name__):
                return getattr(obj, name)()
            fdel = update_meta(__delete__, fdel)

        return property(fget, fset, fdel, doc)
