#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Interface to O2 SMS Connector

using:

    from lib4.smsconnectorO2 import SMSConnectorO2
    con = SMSConnectorO2(your_baID, path_to_cert_file, path_to_key_file)
    res = con.send('+420604238051', 'text of SMS in ASCII or unicode')
    print res
    res = con.receive()
    print res

'''

import urllib
import lib4.text

class SMSConnectorO2(object):
    SERVER_URL = 'https://smsconnector.cz.o2.com/smsconnector/getpost/GP'

    def __init__(self, baID, cert_file, key_file, **kw):
        self.baID = baID
        self.cert_file = cert_file
        self.key_file = key_file
        self.URLopener = urllib.FancyURLopener(key_file=self.key_file,
                                          cert_file=self.cert_file)

    def _parseResponse(self, response):
        res = {}
        key = ''
        for l in response.readlines():
            if '=' in l:
                (key, val) = l.split('=')
                res[key] = val.strip()
            else:
                res[key] += l
        return res



    def send(self, toNumber, text, **kw):
        '''send SMS
           toNumber - destination number of SMS
           text - text of the message in unicode or ASCII

           optional parameters in kw:
           msgID - unique message ID, should not contain semicolon (';')
           deliveryReport - request for delivery Report, default False
           multipart - default False, if True, long message will be divided
                into parts, each 154 chars long.
           validityPeri - validity period of message in seconds. Default 0.
           fromNumb - originating number of SMS. Default baID.
        '''
        params = kw.copy()
        params['action'] = 'send'
        params['baID'] = self.baID
        params['toNumber'] = toNumber
        params['text'] = lib4.text.unicode2ascii(text)
        response = self.URLopener.open(self.SERVER_URL,
                                       urllib.urlencode(params))
        return self._parseResponse(response)

    def receive(self):
        '''receive one SMS from queue.
           If there is no message in queue, returns empty dictionary
        '''
        params = {}
        params['action'] = 'receive'
        params['baID'] = self.baID
        response = self.URLopener.open(self.SERVER_URL,
                                       urllib.urlencode(params))
        response = self._parseResponse(response)
        if response:
            cres = self.confirm(response['msgID'], response['baID'])
        return response

    def confirm(self, refMsgID, refBaID):
        '''confirms receiving of SMS'''
        params = {}
        params['action'] = 'confirm'
        params['baID'] = self.baID
        params['refMsgID'] = refMsgID
        params['refBaID'] = refBaID
        response = self.URLopener.open(self.SERVER_URL,
                                       urllib.urlencode(params))
        return self._parseResponse(response)
