from tokenize import generate_tokens, tok_name
from token import NAME, STRING
from cStringIO import StringIO
import re

__all__ = ['Token', 'getFileEncoding']


coding_search = re.compile(r'coding[:=]\s*([-\w.]+)').search
bom_signature = '\xef\xbb\xbf'


def getFileEncoding(file, default='ascii'):
    fd = open(file, 'rU')

    line_max = 512
    line = fd.readline(line_max)
    if line.startswith(bom_signature):
        return 'utf-8'

    m = coding_search(line)
    if not m:
        line = fd.readline(line_max)
        m = coding_search(line)
        if not m:
            return default
    return m.group(1)



class Token(object):

    __slots__ = ('tokid', 'token', 'begin_ln', 'begin_col', 'end_ln', 'end_col')

    def __init__(self, tokid, token, begin, end):
        self.tokid = tokid
        self.token = token
        self.begin_ln, self.begin_col = begin
        self.end_ln, self.end_col = end

    name = property(lambda s: tok_name[s.tokid])

    def __repr__(self):
        return 'Token(%s[%d], %r, %d.%d-%d.%d)' % (
            self.name, self.tokid, self.token,
            self.begin_ln, self.begin_col,
            self.end_ln, self.end_col,
        )

    def __eq__(self, other):
        return self.tokid == other.tokid and self.token == other.token

    def __ne__(self, other):
        return not self == other



class TokenSequence(list):

    @classmethod
    def from_string(klass, source):
        sio = StringIO(source)
        self = klass()
        if 0:
            for tokid, token, begin, end, line in generate_tokens(sio.readline):
                print 'LINE', repr(line)
                self.append(Token(tokid, token, begin, end))
            return self
        self.extend(
            Token(tokid, token, begin, end)
            for tokid, token, begin, end, line in generate_tokens(sio.readline)
        )
        return self

    @classmethod
    def from_file(klass, filename):
        self = klass()
        self.extend(
            Token(tokid, token, begin, end)
            for tokid, token, begin, end, line in open(filename).readline
        )
        return self

    def getline(self, ln):
        pass

    def iterlines(self):
        pass

    def __setitem__(self, idx, tok):
        pass

    def __delitem__(self, idx):
        pass

    def insert(self, tok):
        pass

    def search(self, tok):
        pass

    def search_seq(self, tok):
        pass





if __name__ == '__main__':

    def p(source):
        for tok in TokenSequence.from_string(source):
            print tok
        print

    p('a + / 2\n')
    p('"""\\\naaa\n  bbb""" + "x"\n')
    p('.a')
