# -*- coding: utf-8 -*-

__all__ = ['get_screen_size']


def get_screen_size():
    '''Returns height and width of terminal window'''

    import os
    import fcntl
    import termios
    import struct

    def try_screen_size(fd):
        try:
            return struct.unpack('hh',fcntl.ioctl(fd,termios.TIOCGWINSZ,'1234'))
        except:
            return None
    for fd in (0, 1, 2):
        size = try_screen_size(fd)
        if size is not None:
            return size
    fd = os.open(os.ctermid(), os.O_RDONLY)
    size = try_screen_size(fd)
    os.close(fd)
    if size is not None:
        return size
    return os.environ.get('LINES'), os.environ.get('COLUMNS')


if __name__ == '__main__':
    x = get_screen_size()
    print x
