import unittest
from lib4.containers import *
from lib4.combinatorics import permutations
from datetime import *


RVM = RangeValueMap


def ranges(sd):
    return sd._ranges


class TestRangeValueMap(unittest.TestCase):

    def test10_DisjunctRanges(self):
        for inp in permutations(
            [(0,0), ((1,10),1), ((11,20),2), ((21,30),3)]
        ):
            res = ranges(RVM(*inp))
            self.assertEqual(
                res, [(0,0,0), (1,10,1), (11,20,2), (21,30,3)],
                'error for input=%s, result=%s' % (inp, res)
            )

    def test11_DisjunctRangesWithTheSameValue(self):
        for inp in permutations(
            [(0,'a'), (1,'a'), ((2,3),'a'), ((4,8),'a'), (9,'a')]
        ):
            res = ranges(RVM(*inp))
            self.assertEqual(
                res, [(0,9,'a')],
                'error for input=%s, result=%s' % (inp, res)
            )

    def test15_ReplaceExistingRange(self):
        sl = RVM(((10,19),'a'))
        sl.setrange(10,19,'A')
        self.assertEqual(ranges(sl), [(10,19,'A')])
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        sl.setrange(10,19,'A')
        self.assertEqual(ranges(sl), [(10,19,'A'), (20,29,'b'), (30,39,'c')])
        sl.setrange(20,29,'B')
        self.assertEqual(ranges(sl), [(10,19,'A'), (20,29,'B'), (30,39,'c')])
        sl.setrange(30,39,'C')
        self.assertEqual(ranges(sl), [(10,19,'A'), (20,29,'B'), (30,39,'C')])

    def test20_ReplaceBeginingOfExisting(self):
        sl = RVM(((10,20), 'a'))
        sl.setrange(10,15,'A')
        self.assertEqual(ranges(sl), [(10,15,'A'),(16,20,'a')])
        sl = RVM(((1,2),'a'))
        sl.setrange(1,1,'A')
        self.assertEqual(ranges(sl), [(1,1,'A'),(2,2,'a')])

    def test21_ReplaceBeginingOfExistingWithTheSameValue(self):
        sl = RVM(((10,20),'a'))
        sl.setrange(10,15,'a')
        self.assertEqual(ranges(sl), [(10,20,'a')])
        sl = RVM(((1,2),'a'))
        sl.setrange(1,1,'a')
        self.assertEqual(ranges(sl), [(1,2,'a')])

    def test25_ReplaceEndOfExisting(self):
        sl = RVM(((10,20),'a'))
        sl.setrange(16,20,'A')
        self.assertEqual(ranges(sl), [(10,15,'a'),(16,20,'A')])
        sl = RVM(((1,2),'a'))
        sl.setrange(2,2,'A')
        self.assertEqual(ranges(sl), [(1,1,'a'),(2,2,'A')])

    def test26_ReplaceEndOfExistingWithTheSameValue(self):
        sl = RVM(((10,20),'a'))
        sl.setrange(16,20,'a')
        self.assertEqual(ranges(sl), [(10,20,'a')])
        sl = RVM(((1,2),'a'))
        sl.setrange(2,2,'a')
        self.assertEqual(ranges(sl), [(1,2,'a')])

    def test30_BeginingOverlay(self):
        sl = RVM(((10,20), 'a'))
        sl.setrange(5,15,'A')
        self.assertEqual(ranges(sl), [(5,15,'A'),(16,20,'a')])
        sl = RVM(((1,2),'a'))
        sl.setrange(0,1,'A')
        self.assertEqual(ranges(sl), [(0,1,'A'),(2,2,'a')])

    def test31_BeginingOverlayWithTheSameValue(self):
        sl = RVM(((10,20), 'a'))
        sl.setrange(5,15,'a')
        self.assertEqual(ranges(sl), [(5,20,'a')])
        sl = RVM(((1,2),'a'))
        sl.setrange(0,1,'a')
        self.assertEqual(ranges(sl), [(0,2,'a')])

    def test35_EndOverlay(self):
        sl = RVM(((10,20), 'a'))
        sl.setrange(15,25,'A')
        self.assertEqual(ranges(sl), [(10,14,'a'),(15,25,'A')])
        sl = RVM(((1,2),'a'))
        sl.setrange(2,3,'A')
        self.assertEqual(ranges(sl), [(1,1,'a'),(2,3,'A')])

    def test36_EndOverlay(self):
        sl = RVM(((10,20), 'a'))
        sl.setrange(15,25,'a')
        self.assertEqual(ranges(sl), [(10,25,'a')])
        sl = RVM(((1,2),'a'))
        sl.setrange(2,3,'a')
        self.assertEqual(ranges(sl), [(1,3,'a')])

    def test40_FillGapWithTheSameValue(self):
        sl = RVM(((10,20),'a'), ((30,40),'a'))
        sl.setrange(21,29,'a')
        self.assertEqual(ranges(sl), [(10,40,'a')])
        sl = RVM((1,'a'), (3,'a'))
        sl.setrange(2,2,'a')
        self.assertEqual(ranges(sl), [(1,3,'a')])

    def test41_FillGapWithTheFirstValue(self):
        sl = RVM(((10,20),'a'), ((30,40),'b'))
        sl.setrange(21,29,'a')
        self.assertEqual(ranges(sl), [(10,29,'a'),(30,40,'b')])
        sl = RVM((1,'a'), (3,'b'))
        sl.setrange(2,2,'a')
        self.assertEqual(ranges(sl), [(1,2,'a'), (3,3,'b')])

    def test42_FillGapWithTheSecondValue(self):
        sl = RVM(((10,20),'a'), ((30,40),'b'))
        sl.setrange(21,29,'b')
        self.assertEqual(ranges(sl), [(10,20,'a'),(21,40,'b')])
        sl = RVM((1,'a'), (3,'b'))
        sl.setrange(2,2,'b')
        self.assertEqual(ranges(sl), [(1,1,'a'), (2,3,'b')])

    def test50(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'))
        sl.setrange(15,25,'c')
        self.assertEqual(ranges(sl), [(10,14,'a'),(15,25,'c'),(26,29,'b')])

    def test51(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        sl.setrange(15,35,'d')
        self.assertEqual(ranges(sl), [(10,14,'a'),(15,35,'d'),(36,39,'c')])

    def test52(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        sl.setrange(15,45,'d')
        self.assertEqual(ranges(sl), [(10,14,'a'),(15,45,'d')])

    def test53(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        sl.setrange(1,45,'d')
        self.assertEqual(ranges(sl), [(1,45,'d')])

    def test60_Get(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        self.assertEqual(sl.get(10), 'a')
        self.assertEqual(sl.get(10, 'x'), 'a')
        self.assertEqual(sl.get(15), 'a')
        self.assertEqual(sl.get(19), 'a')
        self.assertEqual(sl.get(25), 'b')
        self.assertEqual(sl.get(35), 'c')
        self.assertEqual(sl.get(1), None)

    def test61_GetFromEmpty(self):
        sl = RVM()
        self.assertEqual(sl.get(1), None)
        self.assertEqual(sl.get(1, 42), 42)

    def test62_MinMax(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        self.assertEqual(sl.minkey(), 10)
        self.assertEqual(sl.minkey(-1), 10)
        self.assertEqual(sl.maxkey(), 39)
        self.assertEqual(sl.maxkey(-1), 39)

    def test63_MinMaxOfEmpty(self):
        sl = RVM()
        self.assertEqual(sl.minkey(), None)
        self.assertEqual(sl.minkey(-1), -1)
        self.assertEqual(sl.maxkey(), None)
        self.assertEqual(sl.maxkey(1), 1)

    def test70_Items(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        self.assertEqual(
            list(sl.items(10,29)), [((10,19),'a'), ((20,29),'b')]
        )
        self.assertEqual(
            list(sl.items(12,14)), [((12,14),'a')]
        )
        self.assertEqual(
            list(sl.items(1,2)), []
        )
        self.assertEqual(
            list(sl.items(15,35)), [((15,19),'a'), ((20,29),'b'),((30,35),'c')]
        )
        self.assertEqual(
            list(sl.keys(15,35)), [(15,19), (20,29), (30,35)]
        )
        self.assertEqual(
            list(sl.values(15,35)), ['a', 'b', 'c']
        )

    def test71_ItemsOfEmpty(self):
        sl = RVM()
        self.assertEqual(list(sl.items(1,2)), [])
        self.assertEqual(list(sl.items()), [])
        self.assertEqual(list(sl.keys()), [])
        self.assertEqual(list(sl.values()), [])

    def test72_ItemsWithNoSplit(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'), ((30,39),'c'))
        self.assertEqual(
            list(sl.items(10,29,False)), [((10,19),'a'), ((20,29),'b')]
        )
        self.assertEqual(
            list(sl.items(12,14,False)), [((10,19),'a')]
        )
        self.assertEqual(
            list(sl.items(1,2,False)), []
        )
        self.assertEqual(
            list(sl.items(15,35,False)),
            [((10,19),'a'), ((20,29),'b'),((30,39),'c')]
        )
        self.assertEqual(
            list(sl.keys(15,35,False)), [(10,19), (20,29), (30,39)]
        )
        self.assertEqual(
            list(sl.values(15,35,False)), ['a', 'b', 'c']
        )

    def test90_Copy(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'))
        cp = sl.copy()
        self.assertEqual(list(cp.items()), [((10,19),'a'), ((20,29),'b')])

    def test91_EmptyCopy(self):
        sl = RVM()
        cp = sl.copy()
        self.assertEqual(list(cp.items()), [])

    def test92_InplaceUpdate(self):
        sl = RVM(((10,19),'a'), ((20,29),'b'))
        sl.update(RVM(((10,19),'A'), ((30,39),'C')))
        self.assertEqual(
            list(sl.items()), [((10,19),'A'), ((20,29),'b'),((30,39),'C')]
        )


DAY = timedelta(1)

class TestDateRangeValueMap(unittest.TestCase):

    def test10(self):
        inp = [
            (date(2001,1,1), 'a'),
            ((date(2002,1,1),date(2003,1,1)), 'b'),
            ((date(2004,1,1),date(2005,1,1)), 'c'),
            ((date(2006,1,1),date(2007,1,1)), 'd'),
        ]
        for inp in permutations(inp):
            rvm = RVM(*inp, **{'resolution':DAY})
            res = ranges(rvm)
            self.assertEqual(res, [
                (date(2001,1,1), date(2001,1,1), 'a'),
                (date(2002,1,1),date(2003,1,1), 'b'),
                (date(2004,1,1),date(2005,1,1), 'c'),
                (date(2006,1,1),date(2007,1,1), 'd'),
                ], 'error for input=%s, result=%s' % (inp, res)

            )

    def test11(self):
        rvm = RVM(
            ((date(2002,1,1),date(2003,1,1)), 'a'),
            ((date(2004,1,1),date(2005,1,1)), 'b'),
            resolution=DAY
        )
        rvm.setrange(date(2002,6,1),date(2004,6,1),'c')
        self.assertEqual(ranges(rvm), [
            (date(2002,1,1),date(2002,5,31), 'a'),
            (date(2002,6,1),date(2004,6,1), 'c'),
            (date(2004,6,2),date(2005,1,1), 'b'),

        ])

