import unittest
from lib4.content_types import *
import os

THIS_DIR = os.path.dirname(__file__)

GCT = guessContentType
GE = guessExtension

class TestContentTypes(unittest.TestCase):

    def test00_singletons(self):
        ms1 = magicService()
        ms2 = magicService()
        assert ms1 is ms2
        ms1 = mimeService()
        ms2 = mimeService()
        assert ms1 is ms2

    def test01_guessContentTypeOfExistingFile(self):
        F = os.path.join(THIS_DIR, 'samples', 'profile')
        self.assertEqual(GCT(F), 'application/octet-stream')
        self.assertEqual(GCT(F,default='text/plain'), 'text/plain')
        self.assertEqual(GCT(F,path=F).split()[0], 'text/plain')

    def test02_guessContentTypeOfNonExistingFile(self):
        self.assertEqual(GCT('*.txt'), 'text/plain')
        self.assertEqual(GCT('*.txt',default='text/rest'), 'text/plain')
        self.assertEqual(GCT('*.txt',path='*.txt'), 'text/plain')

    def test03_guessContentTypeOfEncodedFile(self):
        self.assertEqual(GCT('*.txt.gz'), 'application/gzip')
        self.assertEqual(GCT('*.txt.gz',default='text/rest'),'application/gzip')
        self.assertEqual(GCT('*.txt.gz',path='*.txt.gz'), 'application/gzip')

    def test04_guessContentTypeWithUnknownExtension(self):
        f = os.path.join(THIS_DIR, 'imgs', 'p.unknown')
        self.assertEqual(GCT(f), 'application/octet-stream')
        self.assertEqual(GCT(f, default='text/rest'), 'text/rest')
        self.assertEqual(GCT(f, path=f), 'image/jpeg')

    def test10_guessExtension(self):
        self.assertEqual(GE('application/octet-stream'), '.bin')
        self.assertEqual(GE('image/gif'), '.gif')
        self.assertEqual(GE('image/jpeg'), '.jpg')
        self.assertEqual(GE('text/plain'), '.txt')
        self.assertEqual(GE('text/plain; charset=us-ascii'), '.txt')
        self.assertEqual(GE('application/msword'), '.doc')
        self.assertEqual(GE('application/word'), '.doc')
        self.assertEqual(GE('application/vnd.ms-excel'), '.xls')
        self.assertEqual(GE('application/vnd.ms-powerpoint'), '.ppt')

    def test20_Problematic1(self):
        F = os.path.join(THIS_DIR, 'samples', 'stanovy.doc')
        self.assertEqual(GCT(F), 'application/msword')
        self.assertEqual(GCT(F,default='text/plain'), 'application/msword')
        self.assertEqual(GCT(F,path=F), 'application/msword')
