import unittest
from lib4.ini_parser import *

class LoggingConfigParser(AbstractConfigParser):

    def __init__(self):
        self.values = []

    def add_setting(self, section, name, value, lineInfo):
        self.values.append((section, name, value, lineInfo))





class TestIniParser(unittest.TestCase):

    def test(self):
        ap = LoggingConfigParser()
        ap.readString('''
var1 = val1

[Section 1]
var1 = val1

[Domlets xmlns:form=http://windmill.cz/form]
{http://windmill.cz/form}domlet = line1
    line2
    line3

    line5

form:domlet = foo.bar


''', 'test.ini')

        self.assertEqual(len(ap.values), 4)

        sec, var, val, line = ap.values[0]
        self.assertEqual(sec, None)
        self.assertEqual(var, 'var1')
        self.assertEqual(val, 'val1')
        self.assertEqual(line[0], 'test.ini')
        self.assertEqual(line[1], 2)

        sec, var, val, line = ap.values[1]
        self.assertEqual(sec, 'Section 1')
        self.assertEqual(var, 'var1')
        self.assertEqual(val, 'val1')
        self.assertEqual(line[0], 'test.ini')
        self.assertEqual(line[1], 5)

        sec, var, val, line = ap.values[2]
        self.assertEqual(sec, 'Domlets xmlns:form=http://windmill.cz/form')
        self.assertEqual(var, '{http://windmill.cz/form}domlet')
        self.assertEqual(val, 'line1\nline2\nline3\n\nline5')
        self.assertEqual(line[0], 'test.ini')
        self.assertEqual(line[1], 8)

        sec, var, val, line = ap.values[3]
        self.assertEqual(sec, 'Domlets xmlns:form=http://windmill.cz/form')
        self.assertEqual(var, 'form:domlet')
        self.assertEqual(val, 'foo.bar')
        self.assertEqual(line[0], 'test.ini')
        self.assertEqual(line[1], 14)

