import unittest
from lib4.iterutils import *


class TestFlatten(unittest.TestCase):

    def test_linearizeStruct(self):
        self.assertEqual(list(flatten((1,))),  [1])
        self.assertEqual(list(flatten([1])),  [1])
        self.assertEqual(list(flatten([1,(2,)])),  [1,2])
        self.assertEqual(list(flatten(['aaa',('bbb',)])),  ['aaa','bbb'])




class TestFirstLast(unittest.TestCase):

    def test10_DefaultPredicate(self):
        self.assertEqual(findFirst(range(10)), 1)

    def test11_ExplicitPredicate(self):
        self.assertEqual(findFirst(range(10), pred=lambda x:x%3==2), 2)

    def test12_DefaultValue(self):
        self.assertEqual(findFirst([0,0,0]), None)
        self.assertEqual(findFirst([0,0,0], default=1), 1)

    def test20_DefaultPredicate(self):
        self.assertEqual(findLast(range(10), reverse=True), 9)

    def test21_ExplicitPredicate(self):
        self.assertEqual(
            findLast(range(10), pred=lambda x:x%3==2, reverse=True), 8
        )

    def test22_DefaultValue(self):
        self.assertEqual(findLast([0,0,0], reverse=True), None)
        self.assertEqual(findLast([0,0,0], reverse=True, default=1), 1)

    def test30_DefaultPredicate(self):
        self.assertEqual(findLast(range(10), reverse=False), 9)

    def test31_ExplicitPredicate(self):
        self.assertEqual(
            findLast(range(10), pred=lambda x:x%3==2, reverse=False), 8
        )

    def test32_DefaultValue(self):
        self.assertEqual(findLast([0,0,0], reverse=False), None)
        self.assertEqual(findLast([0,0,0], reverse=False, default=1), 1)







class TestIterDecorator(unittest.TestCase):

    def test01_ItemAsTuple(self):
        item, = iterDecorator('a')
        i, v = item
        self.assertEqual(i, item)
        self.assertEqual(v, 'a')

    def test10_Empty(self):
        self.assertEqual(list(iterDecorator([])), [])

    def test11_1Element(self):
        i1, = iterDecorator('a')
        self.assertEqual(i1.item, 'a')
        self.assertEqual(i1.index, 0)
        self.assertEqual(i1.absIndex, 0)
        self.assertEqual(i1.isFirst, True)
        self.assertEqual(i1.isLast, True)
        self.assertEqual(i1.isOdd, True)
        self.assertEqual(i1.isEven, False)

    def test12_2Element(self):
        i1,i2 = iterDecorator('ab')
        self.assertEqual(i1.item, 'a')
        self.assertEqual(i1.index, 0)
        self.assertEqual(i1.isFirst, True)
        self.assertEqual(i1.isLast, False)
        self.assertEqual(i1.isOdd, True)
        self.assertEqual(i1.isEven, False)
        self.assertEqual(i2.item, 'b')
        self.assertEqual(i2.index, 1)
        self.assertEqual(i2.isFirst, False)
        self.assertEqual(i2.isLast, True)
        self.assertEqual(i2.isOdd, False)
        self.assertEqual(i2.isEven, True)

    def test13_2Element(self):
        i1,i2,i3 = iterDecorator('abc')
        self.assertEqual(i1.item, 'a')
        self.assertEqual(i1.index, 0)
        self.assertEqual(i1.isFirst, True)
        self.assertEqual(i1.isLast, False)
        self.assertEqual(i1.isOdd, True)
        self.assertEqual(i1.isEven, False)
        self.assertEqual(i2.item, 'b')
        self.assertEqual(i2.index, 1)
        self.assertEqual(i2.isFirst, False)
        self.assertEqual(i2.isLast, False)
        self.assertEqual(i2.isOdd, False)
        self.assertEqual(i2.isEven, True)
        self.assertEqual(i3.item, 'c')
        self.assertEqual(i3.index, 2)
        self.assertEqual(i3.isFirst, False)
        self.assertEqual(i3.isLast, True)
        self.assertEqual(i3.isOdd, True)
        self.assertEqual(i3.isEven, False)

    def test20_Offset(self):
        i1,i2 = iterDecorator('ab', offset=10)
        self.assertEqual(i1.index, 0)
        self.assertEqual(i1.absIndex, 10)
        self.assertEqual(i2.index, 1)
        self.assertEqual(i2.absIndex, 11)

    def test30_DefaultClassifier(self):
        i1,i2 = iterDecorator('ab', offset=10)
        self.assertEqual(i1.itemClass, 'odd')
        self.assertEqual(i2.itemClass, 'even')

    def test31_UserDefinedClassifier(self):
        i1,i2 = iterDecorator(
            'ab', offset=10, classifier=lambda i:i.item.upper())
        self.assertEqual(i1.itemClass, 'A')
        self.assertEqual(i2.itemClass, 'B')







class TestPad(unittest.TestCase):

    def test1_IterableHasNotMinLength(self):
        self.assertEqual(list(pad(range(2), None, 3)), [0,1,None])

    def test2_IterableHasMinLength(self):
        self.assertEqual(list(pad(range(3), None, 3)), [0,1,2])

    def test3_IterableExceedsMinLength(self):
        self.assertEqual(list(pad(range(4), None, 3)), [0,1,2,3])






class TestSequenceToTableVertically(unittest.TestCase):

    cases = [
            (range(0), 3, []),
            (range(1), 3, [[0,None, None]]),
            (range(2), 3, [[0, 1, None]]),
            (range(3), 3, [[0, 1, 2]]),
            (range(4), 3, [[0, 2, 3], [1, None, None]]),
            (range(7), 3, [[0, 3, 5], [1, 4, 6], [2, None, None]]),

            (range(8), 4, [[0, 2, 4, 6],[1, 3, 5, 7]]),
            (range(9), 4, [[0, 3, 5, 7],[1, 4, 6, 8],[2, None, None, None]]),

            (range(3), 1, [[0],[1],[2]]),

            (range(5), 2, [[0, 3], [1, 4], [2, None]]),
            (range(2), 2, [[0, 1]]),
            (range(4), 2, [[0, 2], [1, 3]]),
    ]

    def test(self):
        for seq, cols, result in self.cases:
            tab = list(sequenceToTableVertically(seq, cols))
            self.assertEqual(tab, result)


class TestSequenceToTableHorizontally(unittest.TestCase):

    cases = [
            (range(0), 3, ()),
            (range(1), 3, ((0,None, None),)),
            (range(2), 3, ((0, 1, None),)),
            (range(3), 3, ((0, 1, 2),)),
            (range(4), 3, ((0, 1, 2), (3, None, None))),
            (range(7), 3, ((0, 1, 2), (3, 4, 5), (6, None, None))),

            (range(8), 4, ((0, 1, 2, 3),(4, 5, 6, 7))),
            (range(9), 4, ((0, 1, 2, 3),(4, 5, 6, 7),(8, None, None, None))),

            (range(3), 1, ((0,),(1,),(2,))),

            (range(5), 2, ((0, 1), (2, 3), (4, None))),
            (range(2), 2, ((0, 1),)),
            (range(4), 2, ((0, 1), (2, 3))),
    ]

    def test(self):
        for seq, cols, result in self.cases:
            tab = tuple(sequenceToTableHorizontally(seq, cols))
            self.assertEqual(tab, result)






class N:
    def __init__(self, sorting=None):
        self.sorting=sorting
    def __repr__(self):
        return str(self.sorting)
    def __cmp__(self, other):
        return cmp(self.sorting, other.sorting)

class TestUpdateSortingAttr(unittest.TestCase):

    cases = [
        ([], []),
        ([N(1000)], [1000]),
        ([N(100)], [100]),
        ([N(1)], [100]),
        ([N(1),N(2)], [100,120]),

        ([N()], [1000]),
        ([N(),N()], [1000,1020]),
        ([N(),N(),N()], [1000,1020,1040]),

        ([N(),N(1000)], [980,1000]),
        ([N(),N(),N(1000)], [960,980,1000]),
        ([N(),N(),N(),N(1000)], [940,960,980,1000]),

        ([N(),N(200)], [180,200]),
        ([N(),N(),N(200)], [160,180,200]),

        ([N(),N(100)], [100,120]),
        ([N(),N(),N(100)], [100,120,140]),

        ([N(100),N(),N(101)], [100,120,140]),
        ([N(100),N(),N(),N(101)], [100,120,140,160]),

        ([N(100),N(),N(102)], [100,101,102]),
        ([N(100),N(),N(103)], [100,101,103]),
        ([N(100),N(),N(),N(103)], [100,101,102,103]),
        ([N(100),N(),N(),N(105)], [100,101,102,105]),
        ([N(100),N(),N(104)], [100,102,104]),

        ([N(100),N()], [100,120]),
        ([N(100),N(),N()], [100,120,140]),
        ([N(100),N(),N(),N()], [100,120,140,160]),
    ]


    def test(self):
        for i, (old, new_sort) in enumerate(self.cases):
            res = list(updateSortingAttr(old,key=lambda i:getattr(i,'sorting')))
            res_sort = [s for s,_ in res]
            res_seq = [n for _,n in res]
            for o,n in zip(old, res_seq):
                assert o is n
            self.assertEqual(res_sort, new_sort, 'test %d failed: %r x %r' % (
                (i+1), res_sort, new_sort
            ))

