import unittest
from datetime import datetime, date, timedelta
from lib4.timeutils import *


class TestDate (unittest.TestCase) :

    def test10_FromDate (self) :
        self.assertEqual(Date(date(2000,1,1)), date(2000,1,1))

    def test11_FromDatetime (self) :
        self.assertEqual(Date(datetime(2000,1,1,2,3,4)), date(2000,1,1))

    def test12_FromTuple (self) :
        self.assertEqual(Date(2000,1,1), date(2000,1,1))
        self.assertEqual(Date((2000,1,1)), date(2000,1,1))

    def test13_FromString (self) :
        self.assertEqual(Date('2000-2-1'), date(2000,2,1))
        self.assertEqual(Date('1.2.2000'), date(2000,2,1))

    def test14_Constants (self) :
        self.assertEqual(Date('today'), date.today())

    def test15_Limits (self) :
        self.assertEqual(Date('-infinity'), DATE_MIN)
        self.assertEqual(Date('infinity'), DATE_MAX)



class TestDateTime (unittest.TestCase) :

    def test10_FromDate (self) :
        self.assertEqual(
            Datetime(date(2000,1,1)),
            datetime(2000,1,1).replace(tzinfo=TZLOCAL)
        )

    def test11_FromDatetime (self) :
        self.assertEqual(
            Datetime(datetime(2000,1,1,2,3,4)),
            datetime(2000,1,1,2,3,4).replace(tzinfo=TZLOCAL)
        )

    def test12_FromTuple (self) :
        self.assertEqual(
            Datetime((2000,1,1,2,3,4)),
            datetime(2000,1,1,2,3,4).replace(tzinfo=TZLOCAL)
        )
        self.assertEqual(
            Datetime(2000,1,1,2,3,4),
            datetime(2000,1,1,2,3,4).replace(tzinfo=TZLOCAL)
        )

    def test13_FromString (self) :
        self.assertEqual(
            Datetime('2000-2-1 12:34'),
            datetime(2000,2,1,12,34).replace(tzinfo=TZLOCAL)
        )
        self.assertEqual(
            Datetime('1.2.2000 12:34'),
            datetime(2000,2,1,12,34).replace(tzinfo=TZLOCAL)
        )

    def test14_Constants (self) :
        assert abs(Datetime('now')-Now()) < timedelta(0,1)

    def test15_Limits (self) :
        self.assertEqual(Datetime('-infinity'), DATETIME_MIN)
        self.assertEqual(Datetime('infinity'), DATETIME_MAX)

