# http://pypi.python.org/pypi/bda.intellidatetime/
# http://pypi.python.org/pypi/magicdate/
# http://www.w3.org/TR/2005/NOTE-timezone-20051013/
# http://projects.amor.org/misc/browser/recur.py?format=txt
# http://code-bear.com/code/parsedatetime/
# http://cheeseshop.python.org/pypi/when/
# Actions and Events in Interval Temporal Logic (1994)
# http://citeseer.ist.psu.edu/allen94actions.html

from datetime import datetime, date, timedelta, time
from dateutil import tz, parser

__all__ = [
    'TZLOCAL', 'TZUTC', 'SECOND', 'DAY', 'TIME_RESOLUTION',
    'DATETIME_MIN', 'DATETIME_MAX', 'DATE_MIN', 'DATE_MAX',
    'Now', 'Today', 'Datetime', 'Date', 'Time', 'TimeService',
]


TZLOCAL         = tz.tzlocal()
TZUTC           = tz.tzutc()
SECOND          = timedelta(seconds=1)
DAY             = timedelta(days=1)
TIME_RESOLUTION = timedelta(microseconds=1)
DATETIME_MIN    = datetime.min.replace(tzinfo=TZUTC)
DATETIME_MAX    = datetime.max.replace(tzinfo=TZUTC)
DATE_MIN        = date.min
DATE_MAX        = date.max



def Now():
    return datetime.now().replace(tzinfo=TZLOCAL)

def Today():
    return date.today()


def Datetime(*d, **kw):
    if len(d) > 1:
        r = datetime(*d)
    elif len(d) < 1:
        r = Now()
    else:
        d = d[0]
        if isinstance(d, datetime):
            r = d
        elif isinstance(d, date):
            r = datetime(d.year, d.month, d.day)
        elif isinstance(d, basestring):
            if d == "-infinity":
                r = DATETIME_MIN
            elif d == "infinity":
                r = DATETIME_MAX
            elif d == "now":
                r = Now()
            else:
                d = str(d)
                r = parser.parse(d, dayfirst = d.count('.')>1)
        elif isinstance(d, tuple):
            r = datetime(*d)
        elif isinstance(d, (int,float)):
            r = datetime.fromtimestamp(d)
        else:
            raise ValueError(
                        "invalid datetime argument %r" % d
            )
    if r.tzinfo is None:
        tzinfo = kw.get('tzinfo', TZLOCAL)
        r = r.replace(tzinfo=tzinfo)
    return r

def Time(*args):
    if len(args) == 1:
        first = args[0]
        if isinstance(first, basestring):
            args = map(int, first.split(':'))
        elif isinstance(first, time):
            return first
    return time(*args)


def Date(*d):
    if len(d) > 1:
        r = date(*d)
    elif len(d) < 1:
        r = Today()
    else:
        d = d[0]
        if isinstance(d, datetime):
            r = date(d.year, d.month, d.day)
        elif isinstance(d, date):
            r = d
        elif isinstance(d, basestring):
            if d == "-infinity":
                r = DATE_MIN
            elif d == "infinity":
                r = DATE_MAX
            elif d == "today":
                r = Today()
            else:
                d = str(d)
                r = parser.parse(d, dayfirst = d.count('.')>1)
                r = date(r.year, r.month, r.day)
        elif isinstance(d, tuple):
            r = date(*d)
        else:
            raise ValueError(
                        "invalid datetime argument %r" % d
            )
    return r


class TimeService:

    def now(self):
        return Now()

    def today(self):
        return Today()

    date = staticmethod(Date)
    datetime = staticmethod(Datetime)
    time = staticmethod(Time)
    timedelta = staticmethod(timedelta)
