#!/usr/bin/python

import os

__all__ = ['wait4']

if hasattr(os, 'wait4'):
    wait4 = os.wait4

else:

    import ctypes, resource

    __libc = ctypes.cdll.LoadLibrary('libc.so.6')


    class TIMEVAL(ctypes.Structure):
        _fields_ = [
            ('tv_sec', ctypes.c_int),
            ('tv_usec', ctypes.c_int),
        ]

        def __str__(self):
            return "(%s, %s)" % (self.tv_sec, self.tv_usec)

        def __float__(self):
            return self.tv_sec + self.tv_usec/1000000.0

    class RUSAGE(ctypes.Structure):
        _fields_ = [
            ('ru_utime', TIMEVAL),
            ('ru_stime', TIMEVAL),
            ('ru_maxrss', ctypes.c_long),
            ('ru_ixrss', ctypes.c_long),
            ('ru_idrss', ctypes.c_long),
            ('ru_isrss', ctypes.c_long),
            ('ru_minflt', ctypes.c_long),
            ('ru_majflt', ctypes.c_long),
            ('ru_nswap', ctypes.c_long),
            ('ru_inblock', ctypes.c_long),
            ('ru_oublock', ctypes.c_long),
            ('ru_msgsnd', ctypes.c_long),
            ('ru_msgrcv', ctypes.c_long),
            ('ru_nsignals', ctypes.c_long),
            ('ru_nvcsw', ctypes.c_long),
            ('ru_nivcsw', ctypes.c_long),
        ]

    def wait4(pid, opts):
        status = ctypes.c_int(666)
        u = RUSAGE()
        p = __libc.wait4(pid, ctypes.byref(status), 0, ctypes.byref(u))
        r = resource.struct_rusage([
            float(u.ru_utime),
            float(u.ru_stime),
            u.ru_maxrss,
            u.ru_ixrss,
            u.ru_idrss,
            u.ru_isrss,
            u.ru_minflt,
            u.ru_majflt,
            u.ru_nswap,
            u.ru_inblock,
            u.ru_oublock,
            u.ru_msgsnd,
            u.ru_msgrcv,
            u.ru_nsignals,
            u.ru_nvcsw,
            u.ru_nivcsw,
        ])
        if p == -1:
            raise OSError('wait4 failed')
        return (p, status.value, r)
